@@ -1,12 +1,16 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2014    The R Core Team
+ *  Copyright (C) 1998-2016    The R Core Team
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This header file is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2.1 of the License, or
  *  (at your option) any later version.
  *
+ *  This file is part of R. R is distributed under the terms of the
+ *  GNU General Public License, either Version 2, June 1991 or Version 3,
+ *  June 2007. See doc/COPYRIGHTS for details of the copyright status of R.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -22,12 +26,12 @@
 #ifndef R_RANDOM_H
 #define R_RANDOM_H
 
+#include <R_ext/Boolean.h>
+
 #ifdef  __cplusplus
 extern "C" {
 #endif
 
-#include <R_ext/Boolean.h>
-
 typedef enum {
     WICHMANN_HILL,
     MARSAGLIA_MULTICARRY,
@@ -54,6 +58,7 @@ void GetRNGstate(void);
 void PutRNGstate(void);
 
 double unif_rand(void);
+double R_unif_index(double);
 /* These are also defined in Rmath.h */
 double norm_rand(void);
 double exp_rand(void);
