@@ -1,7 +1,7 @@
 #  File src/library/utils/R/sourceutils.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@ removeSource <- function(fn) {
     recurse <- function(part) {
         if (is.name(part)) return(part)  # handles missing arg, PR#15957
         attr(part, "srcref") <- NULL
+        attr(part, "wholeSrcref") <- NULL
+        attr(part, "srcfile") <- NULL
         if (is.language(part) && is.recursive(part)) {
             for (i in seq_along(part))
             	part[[i]] <- recurse(part[[i]])
@@ -158,7 +160,7 @@ getParseText <- function(parseData, id) {
     	text <- character(nrow(d))
     	blank <- seq_along(text)
     } else
-    	blank <- which(!nzchar(text) | (d$token == "STR_CONST" & grepl("^[[]", text)))
+    	blank <- which(!nzchar(text) | (d$token == "STR_CONST" & startsWith(text, "[")))
     for (i in blank) {
 	lines <- getSrcLines(srcfile, d$line1[i], d$line2[i])
         n <- length(lines)
