@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2015  The R Core Team
+ *  Copyright (C) 1997--2017  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -138,8 +138,7 @@ const static char * const falsenames[] = {
 
 SEXP asChar(SEXP x)
 {
-    if (XLENGTH(x) >= 1) {
-	if (isVectorAtomic(x)) {
+	if (isVectorAtomic(x) && XLENGTH(x) >= 1) {
 	    int w, d, e, wi, di, ei;
 	    char buf[MAXELTSIZE];  /* Probably 100 would suffice */
 
@@ -148,9 +147,9 @@ SEXP asChar(SEXP x)
 		if (LOGICAL(x)[0] == NA_LOGICAL)
 		    return NA_STRING;
 		if (LOGICAL(x)[0])
-		    sprintf(buf, "T");
+		    sprintf(buf, "TRUE");
 		else
-		    sprintf(buf, "F");
+		    sprintf(buf, "FALSE");
 		return mkChar(buf);
 	    case INTSXP:
 		if (INTEGER(x)[0] == NA_INTEGER)
@@ -174,7 +173,6 @@ SEXP asChar(SEXP x)
 	    return x;
 	} else if(TYPEOF(x) == SYMSXP)
 	    return PRINTNAME(x);
-    }
     return NA_STRING;
 }
 
@@ -263,35 +261,35 @@ void InitTypeTables(void) {
 
     /* Type2Table */
     for (int type = 0; type < MAX_NUM_SEXPTYPE; type++) {
-        int j = findTypeInTypeTable(type);
+	int j = findTypeInTypeTable(type);
 
-        if (j != -1) {
-            const char *cstr = TypeTable[j].str;
-            SEXP rchar = PROTECT(mkChar(cstr));
-            SEXP rstr = ScalarString(rchar);
-            MARK_NOT_MUTABLE(rstr);
-            R_PreserveObject(rstr);
-            UNPROTECT(1); /* rchar */
-            SEXP rsym = install(cstr);
+	if (j != -1) {
+	    const char *cstr = TypeTable[j].str;
+	    SEXP rchar = PROTECT(mkChar(cstr));
+	    SEXP rstr = ScalarString(rchar);
+	    MARK_NOT_MUTABLE(rstr);
+	    R_PreserveObject(rstr);
+	    UNPROTECT(1); /* rchar */
+	    SEXP rsym = install(cstr);
 
-            Type2Table[type].cstrName = cstr;
-            Type2Table[type].rcharName = rchar;
-            Type2Table[type].rstrName = rstr;
-            Type2Table[type].rsymName = rsym;
-        } else {
-            Type2Table[type].cstrName = NULL;
-            Type2Table[type].rcharName = NULL;
-            Type2Table[type].rstrName = NULL;
-            Type2Table[type].rsymName = NULL;
-        }
+	    Type2Table[type].cstrName = cstr;
+	    Type2Table[type].rcharName = rchar;
+	    Type2Table[type].rstrName = rstr;
+	    Type2Table[type].rsymName = rsym;
+	} else {
+	    Type2Table[type].cstrName = NULL;
+	    Type2Table[type].rcharName = NULL;
+	    Type2Table[type].rstrName = NULL;
+	    Type2Table[type].rsymName = NULL;
+	}
     }
 }
 
 SEXP type2str_nowarn(SEXPTYPE t) /* returns a CHARSXP */
 {
     if (t < MAX_NUM_SEXPTYPE) { /* FIXME: branch not really needed */
-        SEXP res = Type2Table[t].rcharName;
-        if (res != NULL) return res;
+	SEXP res = Type2Table[t].rcharName;
+	if (res != NULL) return res;
     }
     return R_NilValue;
 }
@@ -300,7 +298,7 @@ SEXP type2str(SEXPTYPE t) /* returns a CHARSXP */
 {
     SEXP s = type2str_nowarn(t);
     if (s != R_NilValue) {
-        return s;
+	return s;
     }
     warning(_("type %d is unimplemented in '%s'"), t, "type2str");
     char buf[50];
@@ -311,8 +309,8 @@ SEXP type2str(SEXPTYPE t) /* returns a CHARSXP */
 SEXP type2rstr(SEXPTYPE t) /* returns a STRSXP */
 {
     if (t < MAX_NUM_SEXPTYPE) { /* FIXME: branch not really needed */
-        SEXP res = Type2Table[t].rstrName;
-        if (res != NULL) return res;
+	SEXP res = Type2Table[t].rstrName;
+	if (res != NULL) return res;
     }
     error(_("type %d is unimplemented in '%s'"), t,
 	  "type2ImmutableScalarString");
@@ -322,8 +320,8 @@ SEXP type2rstr(SEXPTYPE t) /* returns a STRSXP */
 const char *type2char(SEXPTYPE t) /* returns a char* */
 {
     if (t < MAX_NUM_SEXPTYPE) { /* FIXME: branch not really needed */
-        const char * res = Type2Table[t].cstrName;
-        if (res != NULL) return res;
+	const char * res = Type2Table[t].cstrName;
+	if (res != NULL) return res;
     }
     warning(_("type %d is unimplemented in '%s'"), t, "type2char");
     static char buf[50];
@@ -335,10 +333,10 @@ const char *type2char(SEXPTYPE t) /* returns a char* */
 SEXP NORET type2symbol(SEXPTYPE t)
 {
     if (t >= 0 && t < MAX_NUM_SEXPTYPE) { /* FIXME: branch not really needed */
-        SEXP res = Type2Table[t].rsymName;
-        if (res != NULL) {
-            return res;
-        }
+	SEXP res = Type2Table[t].rsymName;
+	if (res != NULL) {
+	    return res;
+	}
     }
     error(_("type %d is unimplemented in '%s'"), t, "type2symbol");
 }
@@ -615,6 +613,19 @@ static void isort_with_index(int *x, int *indx, int n)
 }
 
 
+// body(x) without attributes "srcref", "srcfile", "wholeSrcref" :
+// NOTE: Callers typically need  PROTECT(R_body_no_src(.))
+SEXP R_body_no_src(SEXP x) {
+    SEXP b = PROTECT(duplicate(BODY_EXPR(x)));
+    /* R's removeSource() works *recursively* on the body()
+       in  ../library/utils/R/sourceutils.R  but that seems unneeded (?) */
+    setAttrib(b, R_SrcrefSymbol, R_NilValue);
+    setAttrib(b, R_SrcfileSymbol, R_NilValue);
+    setAttrib(b, R_WholeSrcrefSymbol, R_NilValue);
+    UNPROTECT(1);
+    return b;
+}
+
 /* merge(xinds, yinds, all.x, all.y) */
 /* xinds, yinds are along x and y rows matching into the (numeric)
    common indices, with 0 for non-matches.
@@ -653,8 +664,12 @@ SEXP attribute_hidden do_merge(SEXP call, SEXP op, SEXP args, SEXP rho)
     isort_with_index(INTEGER(yi), iy, ny);
 
     /* 1. determine result sizes */
-    for (i = 0; i < nx; i++) if (INTEGER(xi)[i] > 0) break; nx_lone = i;
-    for (i = 0; i < ny; i++) if (INTEGER(yi)[i] > 0) break; ny_lone = i;
+    for (i = 0; i < nx; i++)
+	if (INTEGER(xi)[i] > 0) break;
+    nx_lone = i;
+    for (i = 0; i < ny; i++)
+	if (INTEGER(yi)[i] > 0) break;
+    ny_lone = i;
     double dnans = 0;
     for (i = nx_lone, j = ny_lone; i < nx; i = nnx, j = nny) {
 	int tmp = INTEGER(xi)[i];
@@ -1016,11 +1031,38 @@ const char *getTZinfo(void)
     const char *p = getenv("TZ");
     if(p) return p;
 #ifdef HAVE_REALPATH
-    // This works on Linux, OS X and *BSD: other known OSes set TZ.
-    static char abspath[PATH_MAX+1] = "";
-    if(abspath[0]) return abspath + 20;
-    if(realpath("/etc/localtime", abspath))
-	return abspath + 20; // strip prefix of /usr/share/zoneinfo/
+    // This works on Linux, macOS and *BSD: other known OSes set TZ.
+    static char def_tz[PATH_MAX+1] = "";
+    if(def_tz[0]) return def_tz;
+
+    char abspath[PATH_MAX + 1];
+    const char* lt = realpath("/etc/localtime", abspath);
+# ifdef __APPLE__
+    // macOS <= 10.12 expands to /usr/share/zoneinfo/<zone name>
+    // macOS 10.13 expands to /usr/share/zoneinfo.default/<zone name>
+    // but 10.13.1 expands to /private/var/db/timezone/tz/2017c.1.0/zoneinfo/<zone name>
+    if(lt) {
+	if(strstr(abspath, ".default/"))
+	    strncpy(def_tz, abspath + 28, PATH_MAX);
+	else {
+	    // So guess is of the form .../zoneinfo/<real tz>
+	    p = strstr(abspath, "/zoneinfo/");
+	    if(p) {
+		strncpy(def_tz, p + 10, PATH_MAX);
+	    } else {
+		warning("system timezone name is unknown: set environment variable TZ");
+		return "unknown";
+	    }
+	}
+//	printf("abspath = %s\n", abspath); printf("def_tz = %s\n", def_tz);
+	return def_tz;
+    }
+# else
+    if(lt) {
+	strncpy(def_tz, abspath + 20, PATH_MAX); // strip "/usr/share/zoneinfo/"
+	return def_tz;
+    }
+# endif
 #endif
     warning("system timezone name is unknown: set environment variable TZ");
     return "unknown";
@@ -1369,6 +1411,39 @@ Rboolean utf8Valid(const char *str)
     return valid_utf8(str, strlen(str)) == 0;
 }
 
+SEXP attribute_hidden do_validUTF8(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    checkArity(op, args);
+    SEXP x = CAR(args);
+    if (!isString(x))
+	error(_("invalid '%s' argument"), "x");
+    R_xlen_t n = XLENGTH(x);
+    SEXP ans = allocVector(LGLSXP, n); // no allocation below
+    int *lans = LOGICAL(ans);
+    for (R_xlen_t i = 0; i < n; i++)
+	lans[i] = utf8Valid(CHAR(STRING_ELT(x, i)));
+    return ans;
+}
+
+SEXP attribute_hidden do_validEnc(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    checkArity(op, args);
+    SEXP x = CAR(args);
+    if (!isString(x))
+	error(_("invalid '%s' argument"), "x");
+    R_xlen_t n = XLENGTH(x);
+    SEXP ans = allocVector(LGLSXP, n); // no allocation below
+    int *lans = LOGICAL(ans);
+    for (R_xlen_t i = 0; i < n; i++) {
+	SEXP p = STRING_ELT(x, i);
+	if (IS_BYTES(p) || IS_LATIN1(p)) lans[i] = 1;
+	else if (IS_UTF8(p) || utf8locale) lans[i] = utf8Valid(CHAR(p));
+	else if(mbcslocale) lans[i] = mbcsValid(CHAR(p));
+	else lans[i] = 1;
+    }
+    return ans;
+}
+
 
 /* MBCS-aware versions of common comparisons.  Only used for ASCII c */
 char *Rf_strchr(const char *s, int c)
@@ -1644,7 +1719,16 @@ double R_strtod5(const char *str, char **endptr, char dec,
 	    for (n = 0; *p >= '0' && *p <= '9'; p++) n = (n < MAX_EXPONENT_PREFIX) ? n * 10 + (*p - '0') : n;
 	    if (ans != 0.0) { /* PR#15976:  allow big exponents on 0 */
 		expn += expsign * n;
-		if(exph > 0) expn -= exph;
+		if(exph > 0) {
+		    if (expn - exph < -122) {	/* PR#17199:  fac may overflow below if expn - exph is too small.  
+		                                   2^-122 is a bit bigger than 1E-37, so should be fine on all systems */
+		    	for (n = exph, fac = 1.0; n; n >>= 1, p2 *= p2)
+			    if (n & 1) fac *= p2;
+			ans /= fac;
+			p2 = 2.0;
+		    } else
+			expn -= exph;
+		}
 		if (expn < 0) {
 		    for (n = -expn, fac = 1.0; n; n >>= 1, p2 *= p2)
 			if (n & 1) fac *= p2;
@@ -1739,7 +1823,7 @@ SEXP attribute_hidden do_enc2(SEXP call, SEXP op, SEXP args, SEXP env)
     check1arg(args, call, "x");
 
     if (!isString(CAR(args)))
-	errorcall(call, "argumemt is not a character vector");
+	errorcall(call, "argument is not a character vector");
     ans = CAR(args);
     for (i = 0; i < XLENGTH(ans); i++) {
 	el = STRING_ELT(ans, i);
@@ -1766,7 +1850,7 @@ SEXP attribute_hidden do_enc2(SEXP call, SEXP op, SEXP args, SEXP env)
 #ifdef USE_ICU
 # include <locale.h>
 #ifdef USE_ICU_APPLE
-/* Mac OS X is missing the headers */
+/* macOS is missing the headers */
 typedef int UErrorCode; /* really an enum these days */
 struct UCollator;
 typedef struct UCollator UCollator;
@@ -1992,7 +2076,7 @@ SEXP attribute_hidden do_ICUget(SEXP call, SEXP op, SEXP args, SEXP rho)
     checkArity(op, args);
 
     if (collationLocaleSet == 2) {
-        ans = "ASCII";
+	ans = "ASCII";
     } else if(collator) {
 	UErrorCode  status = U_ZERO_ERROR;
 	int type = asInteger(CAR(args));
@@ -2019,7 +2103,7 @@ int Scollate(SEXP a, SEXP b)
 	if (strcmp("C", getLocale()) ) {
 #else
 	const char *p = getenv("R_ICU_LOCALE");
-        if(p && p[0]) {
+	if(p && p[0]) {
 #endif
 	    UErrorCode status = U_ZERO_ERROR;
 	    uloc_setDefault(getLocale(), &status);
@@ -2176,27 +2260,44 @@ SEXP attribute_hidden do_tabulate(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP in = CAR(args), nbin = CADR(args);
     if (TYPEOF(in) != INTSXP)  error("invalid input");
     R_xlen_t n = XLENGTH(in);
-    /* FIXME: could in principle be a long vector */
     int nb = asInteger(nbin);
     if (nb == NA_INTEGER || nb < 0)
 	error(_("invalid '%s' argument"), "nbin");
-    SEXP ans = allocVector(INTSXP, nb);
-    int *x = INTEGER(in), *y = INTEGER(ans);
-    if (nb) memset(y, 0, nb * sizeof(int));
-    for(R_xlen_t i = 0 ; i < n ; i++)
-	if (x[i] != NA_INTEGER && x[i] > 0 && x[i] <= nb) y[x[i] - 1]++;
+    int *x = INTEGER(in);
+    SEXP ans;
+#ifdef LONG_VECTOR_SUPPORT
+    if (n > INT_MAX) {
+	ans = allocVector(REALSXP, nb);
+	double *y = REAL(ans);
+	if (nb) memset(y, 0, nb * sizeof(double));
+	for(R_xlen_t i = 0 ; i < n ; i++)
+	    if (x[i] != NA_INTEGER && x[i] > 0 && x[i] <= nb) y[x[i] - 1]++;
+    } else
+#endif
+    {
+	ans = allocVector(INTSXP, nb);
+	int *y = INTEGER(ans);
+	if (nb) memset(y, 0, nb * sizeof(int));
+	for(R_xlen_t i = 0 ; i < n ; i++)
+	    if (x[i] != NA_INTEGER && x[i] > 0 && x[i] <= nb) y[x[i] - 1]++;
+    }
     return ans;
 }
 
-/* x can be a long vector but xt cannot since the result is integer */
+/* Note: R's findInterval( x , vec, ...)  has first two arguments swapped !
+ * .Internal(findInterval(vec, x, rightmost.closed, all.inside,  left.open))
+ *                         xt  x    right             inside       leftOp
+ * x can be a long vector but xt cannot since the result is integer
+*/
 SEXP attribute_hidden do_findinterval(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
-    SEXP xt, x, right, inside;
+    SEXP xt, x, right, inside, leftOp;
     xt = CAR(args); args = CDR(args);
     x = CAR(args); args = CDR(args);
     right = CAR(args); args = CDR(args);
-    inside = CAR(args);
+    inside = CAR(args);args = CDR(args);
+    leftOp = CAR(args);
     if(TYPEOF(xt) != REALSXP || TYPEOF(x) != REALSXP) error("invalid input");
 #ifdef LONG_VECTOR_SUPPORT
     if (IS_LONG_VEC(xt))
@@ -2205,7 +2306,7 @@ SEXP attribute_hidden do_findinterval(SEXP call, SEXP op, SEXP args, SEXP rho)
     int n = LENGTH(xt);
     if (n == NA_INTEGER) error(_("invalid '%s' argument"), "vec");
     R_xlen_t nx = XLENGTH(x);
-    int sr = asLogical(right), si = asLogical(inside);
+    int sr = asLogical(right), si = asLogical(inside), lO = asLogical(leftOp);
     if (sr == NA_INTEGER)
 	error(_("invalid '%s' argument"), "rightmost.closed");
     if (si == NA_INTEGER)
@@ -2214,10 +2315,11 @@ SEXP attribute_hidden do_findinterval(SEXP call, SEXP op, SEXP args, SEXP rho)
     double *rxt = REAL(xt), *rx = REAL(x);
     int ii = 1;
     for(int i = 0; i < nx; i++) {
-	if (ISNAN(REAL(x)[i])) ii = NA_INTEGER;
+	if (ISNAN(rx[i]))
+	    ii = NA_INTEGER;
 	else {
-	    int mfl = si;
-	    ii = findInterval(rxt, n, rx[i], sr, si, ii, &mfl);
+	    int mfl;
+	    ii = findInterval2(rxt, n, rx[i], sr, si, lO, ii, &mfl); // -> ../appl/interv.c
 	}
 	INTEGER(ans)[i] = ii;
     }
@@ -2270,7 +2372,7 @@ SEXP attribute_hidden do_pretty(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 /*
     r <- .Internal(formatC(x, as.character(mode), width, digits,
-                   as.character(format), as.character(flag), i.strlen))
+		   as.character(format), as.character(flag), i.strlen))
 */
 
 static void
